/*
    Copyright (C) 2005-2009  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _NNTPGRAB_H_
#define _NNTPGRAB_H_

#include "nntpgrab_types.h"

#ifdef  __cplusplus
extern "C" {
#endif

#define NNTPGRAB_API_VERSION    20090505

typedef struct NNTPGrabCore NNTPGrabCore;
typedef struct NNTPGrabCoreClass NNTPGrabCoreClass;

/**
 * Initialize the NNTPGrab Core library. This function needs to be called before any
 * other NNTPGrab function can be called
 *
 * @param version       Needs to be NNTPGRAB_API_VERSION, this is used to prevent version conflicts
 * @param err           Pointer to a char*. If an errors occurs, the reason will be placed in this field. Needs to be freed using ngfree(). Can be NULL to ignore errors
 * @param warnings      Pointer to a char*. If an (non-fatal) warning occurs, the reason will be placed in this field. Needs to be freed using ngfree(). Can be NULL to ignore errors
 * @return              An instance of the NNTPGrabCore which need to be used for other NNTPGrab functions. If this value is NULL, an error has occured (the reason is saved in the err variable)
 *
 * @note                Even if this function returns non-NULL, the warnings argument MAY be set
 */
NNTPGrabCore   *nntpgrab_core_init(int version, char **err, char **warnings);

/**
 * Cleanup the NNTPGrab Core library. After this function is called, the NNTPGrabCore instance can't be used anymore
 *
 * @param obj           An instance of the NNTPGrabCore
 */
void            nntpgrab_core_cleanup(NNTPGrabCore *obj);

/**
 * Retrieve a list of the available usenet servers
 *
 * @param obj           An instance of the NNTPGrabCore
 * @return              A list of all the available usenet servers. Needs to be freed using nntpgrab_config_free_avail_servers()
 */
NGList         *nntpgrab_core_config_get_avail_servers(NNTPGrabCore *obj);

/**
 * Free the list of available usenet servers
 *
 * @param obj           An instance of the NNTPGrabCore
 * @param servers       A list of the available usenet servers as returned from the function nntpgrab_config_get_avail_servers()
 */
void            nntpgrab_core_config_free_avail_servers(NNTPGrabCore *obj, NGList *servers);

/**
 * Get the configuration details about a specific usenet server
 *
 * @param obj           An instance of the NNTPGrabCore
 * @param servername    The name of the server whose details should be retrieved
 * @param server        The address of a ConfigServer structure which will be used to fill in the configuration details
 * @return              TRUE on success (server will be filled in), FALSE if the server isn't known
 */
ngboolean       nntpgrab_core_config_get_server_info(NNTPGrabCore *obj, const char *servername, ConfigServer *server);

/**
 * Add a new usenet server to the NNTPGrab configuration
 *
 * @param obj           An instance of the NNTPGrabCore
 * @param new_server    A structure containing the details about the new usenet server
 * @param errmsg        Pointer to a char*. If an errors occurs, the reason will be placed in this field. Needs to be freed using ngfree(). Can be NULL to ignore errors
 * @return              TRUE when the server was successfully added. FALSE on failure (the reason will be placed in the errmsg field)
 */
ngboolean       nntpgrab_core_config_add_server(NNTPGrabCore *obj, ConfigServer new_server, char **errmsg);

/**
 * Remove a usenet server from the NNTPGrab configuration
 *
 * @param obj           An instance of the NNTPGrabCore
 * @param servername    The name of the server whose entry should be removed
 * @param errmsg        Pointer to a char*. If an errors occurs, the reason will be placed in this field. Needs to be freed using ngfree(). Can be NULL to ignore errors
 * @return              TRUE when the server was successfully removed. FALSE on failure (the reason will be placed in the errmsg field)
 */
ngboolean       nntpgrab_core_config_del_server(NNTPGrabCore *obj, const char *servername, char **errmsg);

/**
 * Adjust the configuration details of an usenet server
 *
 * @param obj           An instance of the NNTPGrabCore
 * @param servername    The name of the server whose entry should be adjusted
 * @param server        A structure containing the new configuration details of the given servername
 * @param errmsg        Pointer to a char*. If an errors occurs, the reason will be placed in this field. Needs to be freed using ngfree(). Can be NULL to ignore errors
 * @return              TRUE when the server was successfully adjusted. FALSE on failure (the reason will be placed in the errmsg field)
 */
ngboolean       nntpgrab_core_config_edit_server(NNTPGrabCore *obj, const char *servername, ConfigServer server, char **errmsg);

/**
 * Retrieve the general configuration options
 *
 * @param obj           An instance of the NNTPGrabCore
 * @return              A structure containing the general configuration options
 */
ConfigOpts      nntpgrab_core_config_get_opts(NNTPGrabCore *obj);

/**
 * Adjust the general configuration options
 *
 * @param obj           An instance of the NNTPGrabCore
 * @param opts          A structure containing the new general configuration options
 */
void            nntpgrab_core_config_set_opts(NNTPGrabCore *obj, ConfigOpts opts);

/**
 * Retrieves a list containing all the sub-folders which are in the given folder
 *
 * @param obj           An instance of the NNTPGrabCore
 * @param parent        The name of the directory whose sub-folders should be retrieved
 * @param folders       Pointer to a NGList*. The list of sub-folders will be placed in this field. This list needs to be free'd using the function nntpgrab_config_free_folder_listing()
 * @return              TRUE on sucess, FALSE if the given directory isn't found or readable
 */
ngboolean       nntpgrab_core_config_get_folder_listing(NNTPGrabCore *obj, const char *parent, NGList **folders);

/**
 * Free a list of sub-folders
 *
 * @param obj           An instance of the NNTPGrabCore
 * @param folders       A list containing subfolders as returned by the function nntpgrab_config_get_folder_listing()
 */
void            nntpgrab_core_config_free_folder_listing(NNTPGrabCore *obj, NGList *folders);

/**
 * Start the NNTPGrab schedular
 *
 * @param obj           An instance of the NNTPGrabCore
 * @return              TRUE is the schedular was successfully started, FALSE if the schedular was already running or in the 'stopping' state
 */
ngboolean       nntpgrab_core_schedular_start(NNTPGrabCore *obj);

/**
 * Stop the NNTPGrab schedular
 *
 * @param obj           An instance of the NNTPGrabCore
 * @param wait          Whether this function should wait until the schedular really has stopped
 * @return              TRUE is schedular was successfully stopped, FALSE if the schedular was already stopped or the in 'stopping' state
 */
ngboolean       nntpgrab_core_schedular_stop(NNTPGrabCore *obj, ngboolean wait);

/**
 * Retrieve the current state of the schedular
 *
 * @param obj           An instance of the NNTPGrabCore
 * @return              The current state of the schedular
 */
SchedularState  nntpgrab_core_schedular_get_state(NNTPGrabCore *obj);

/**
 * Add a new task to the download queue
 *
 * @param obj               An instance of the NNTPGrabCore
 * @param collection_name   The name of the collection in which this task should be added. If there is no collection in the download queue with the given name, it will automatically be created
 * @param subject           The subject of the file
 * @param poster            The name of the poster
 * @param stamp             The UNIX timestamp of the post date
 * @param file_size         The size of the file
 * @param groups            A list containing the newsgroups in which this file is posted
 * @param parts             A list containing the message-id's of all the individual parts belonging to this file
 * @param errmsg            Pointer to a char*. If an errors occurs, the reason will be placed in this field. Needs to be freed using ngfree(). Can be NULL to ignore errors
 * @return                  TRUE if the task was succesfully added, FALSE if an error occured (errmsg will be set)
 */
ngboolean       nntpgrab_core_schedular_add_task_to_queue(NNTPGrabCore *obj, const char *collection_name, const char *subject, const char *poster, time_t stamp, nguint64 file_size, NGList *groups, NGList *parts, char **errmsg);

/**
 * Remove a task from the download queue
 *
 * @param obj               An instance of the NNTPGrabCore
 * @param collection_name   The name of the collection in which the task resides
 * @param subject           The subject of the file
 * @param errmsg            Pointer to a char*. If an errors occurs, the reason will be placed in this field. Needs to be freed using ngfree(). Can be NULL to ignore errors
 * @return                  TRUE if the task was successfully removed, FALSE if an error occured (errmsg will be set)
 */
ngboolean       nntpgrab_core_schedular_del_task_from_queue(NNTPGrabCore *obj, const char *collection_name, const char *subject, char **errmsg);

/**
 * Restart a task in the download queue
 *
 * @param obj               An instance of the NNTPGrabCore
 * @param collection_name   The name of the collection in which the task resides
 * @param subject           The subject of the file
 * @param errmsg            Pointer to a char*. If an errors occurs, the reason will be placed in this field. Needs to be freed using ngfree(). Can be NULL to ignore errors
 * @return                  TRUE if the task was successfully removed, FALSE if an error occured (errmsg will be set)
 */
ngboolean       nntpgrab_core_schedular_restart_task(NNTPGrabCore *obj, const char *collection_name, const char *subject, char **errmsg);

/**
 * Save any recent changes in the download queue to the disk
 *
 * @param obj           An instance of the NNTPGrabCore
 * @param errmsg        Pointer to a char*. If an errors occurs, the reason will be placed in this field. Needs to be freed using ngfree(). Can be NULL to ignore errors
 * @return              TRUE if the task was successfully removed, FALSE if an error occured (errmsg will be set)
 *
 * @note                The download queue will automatically be saved every now and then by the NNTPGrab Core, but it is recommended to perform a manual save as soon as a lot of file have been added to the download queue (like directly after an NZB import)
 */
ngboolean       nntpgrab_core_schedular_save_queue(NNTPGrabCore *obj, char **errmsg);

/**
 * Retrieve a list of all the items in the download queue (using callback functions)
 *
 * @param obj               An instance of the NNTPGrabCore
 * @param collection_func   The function which should be called for every collection in the download queue
 * @param file_func         The function which should be called for every file in the download queue
 * @param group_func        The function which should be called for every group in every file in the download queue
 * @param data              Pointer to some data which will be made available in the callback functions
 */
void            nntpgrab_core_schedular_foreach_task(NNTPGrabCore *obj, ForeachCollectionFunc collection_func, ForeachFileFunc file_func, ForeachGroupFunc group_func, void *data);

/**
 * Move a task in the download queue
 *
 * @param obj                   An instance of the NNTPGrabCore
 * @param collection_name_src   The name of the collection where the subject is part of
 * @param subject_src           The name of the subject which needs to be moved
 * @param collection_name_dest  The name of the collection where the subject needs to be moved to (if NULL, the task will stay in the same collection as it is now)
 * @param position_dest         The position in collection_name_dest where the task needs to be placed at
 * @return                      TRUE on success, FALSE if the collection_name_src or subject_src could not be found
 */
ngboolean       nntpgrab_core_schedular_move_task(NNTPGrabCore *obj, const char *collection_name_src, const char *subject_src, const char *collection_name_dest, int position_dest);

/**
 * Move the position of a collection in the download queue
 *
 * @param obj               An instance of the NNTPGrabCore
 * @param collection_name   The name of the collection which needs to be moved
 * @param new_position      The position in the download queue where the collection needs to be moved to
 * @return                  TRUE on success, FALSE is the collection_name could not be found
 */
ngboolean       nntpgrab_core_schedular_move_collection(NNTPGrabCore *obj, const char *collection_name, int new_position);

#ifdef  __cplusplus
}
#endif

#endif /* _NNTPGRAB_H_ */
